SystemOrganization addCategory: #'MEG-UI'!

Object subclass: #MEGAgentUI
	instanceVariableNames: 'lastMovementReport agent availableHazards'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MEG-UI'!

!MEGAgentUI class methodsFor: 'instance creation' stamp: 'SergioFedi 7/4/2016 12:56'!
agent: aMEGAgent

	^self new
		agent: aMEGAgent;
		yourself.! !

!MEGAgentUI class methodsFor: 'example' stamp: 'SergioFedi 7/4/2016 13:45'!
example
	| world jugador1 ui |
	world := MEGWorldFactory new newWorld.
	jugador1 := world agentNamed: 'Jugador 1'.
	ui := self agent: jugador1.
	jugador1 hazardHandler: (self newGUIHazardHandlerForUI: ui).
	ui availableHazards: MEGHazardFactory new newHazards.

	jugador1
		recruit: 'Frodo' at: 'Bag End' inCompany: 'Hobbits';
		recruit: 'Sam Gamgee' at: 'Bag End' inTheSameCompanyAs: 'Frodo';
		recruit: 'Pippin' at: 'Bag End' inTheSameCompanyAs: 'Sam Gamgee';
		recruit: 'Merry' at: 'Bag End' inCompany: 'Hobbits'.

	^ ui.! !

!MEGAgentUI class methodsFor: 'utilities' stamp: 'SergioFedi 7/5/2016 09:53'!
newGUIHazardHandlerForUI: aMEGAgentUI

	| previousMoveAction hazardsPlayedOnPreviousMoveAction |
	previousMoveAction := nil.
	hazardsPlayedOnPreviousMoveAction := nil.
	^[ :moveAction |
		| hazards |
		(moveAction = previousMoveAction)
			ifTrue: [ hazardsPlayedOnPreviousMoveAction := hazardsPlayedOnPreviousMoveAction + 1 ]
			ifFalse: [ previousMoveAction := moveAction. hazardsPlayedOnPreviousMoveAction := 1 ].
		hazards := aMEGAgentUI availableHazards select: [ :hazard | hazard canApplyToSitePath: moveAction path ].
		hazards isEmpty
			ifTrue: [ nil ]
			ifFalse: [
				(hazardsPlayedOnPreviousMoveAction > 2) ifTrue: [ nil ]
				ifFalse: [hazards atRandom]
			].
	].! !

!MEGAgentUI methodsFor: 'public' stamp: 'SergioFedi 7/4/2016 12:58'!
agent
	^ agent! !

!MEGAgentUI methodsFor: 'private' stamp: 'SergioFedi 7/5/2016 09:53'!
agent: aMEGAgent

	agent := aMEGAgent.! !

!MEGAgentUI methodsFor: 'public' stamp: 'SergioFedi 7/5/2016 09:53'!
availableHazards
	^ availableHazards! !

!MEGAgentUI methodsFor: 'private' stamp: 'SergioFedi 7/4/2016 13:46'!
availableHazards: aCollection

	availableHazards := aCollection.! !

!MEGAgentUI methodsFor: 'initialization' stamp: 'SergioFedi 7/5/2016 09:54'!
initialize

	super initialize.

	agent := nil.
	availableHazards := OrderedCollection new.
	lastMovementReport := ''.! !

!MEGAgentUI methodsFor: 'public' stamp: 'SergioFedi 7/4/2016 13:25'!
move

	self agent companies do: [ :company |
		self moveCompany: company.		
	].! !

!MEGAgentUI methodsFor: 'private' stamp: 'SergioFedi 7/4/2016 14:23'!
moveCompany: aMEGCompany

	| possibleSites chosenSite result |
	lastMovementReport := ''.
	possibleSites := OrderedCollection with: 'None (don''t move company)'.
	possibleSites addAll: (aMEGCompany site neighbours collect: [:each | each name]).
	chosenSite := UIManager default 
		chooseDropList: (self textForCompany: aMEGCompany)
		title: 'Choose site to move to'
		list: possibleSites.
	(chosenSite = possibleSites first) ifTrue: [ ^self ].
	result := self agent move: aMEGCompany to: chosenSite.
	result isSuccess
		ifTrue: [ lastMovementReport := lastMovementReport, (result object at: 'hazardReport') ]
		ifFalse: [ self error: 'Something TERRIBLE happened :o' ].
	! !

!MEGAgentUI methodsFor: 'private' stamp: 'SergioFedi 7/4/2016 17:19'!
printFactionAvailabilityFor: aMEGCompany on: aWriteStream

	| recruitableFactions |
	recruitableFactions := self agent world factions select: [ :each | each isRecruitableIn: aMEGCompany site name ].
	recruitableFactions notEmpty ifTrue: [ 
		aWriteStream
			nextPutAll: ('can recruit: ', (self stringOfNamesOf: recruitableFactions));
			cr.
	].! !

!MEGAgentUI methodsFor: 'public' stamp: 'SergioFedi 7/4/2016 16:31'!
status
	| status |
	status := WriteStream on: String new.
	lastMovementReport notEmpty
		ifTrue: [ status
				nextPutAll: lastMovementReport;
				cr ].
	self agent companies
		do: [ :company | 
			status
				nextPutAll: (self textForCompany: company);
				cr;
				nextPutAll: 'in ';
				nextPutAll: company site name;
				cr.
			self printFactionAvailabilityFor: company on: status ]
		separatedBy: [ status cr ].
	^ status contents! !

!MEGAgentUI methodsFor: 'private' stamp: 'SergioFedi 7/4/2016 13:21'!
stringOfNamesOf: aCollection

	| allButLast |
	allButLast := aCollection allButLast.
	^allButLast isEmpty
		ifTrue: [ aCollection last name ]
		ifFalse: [
			| status |
			status := WriteStream on: String new.
			allButLast do: [ :each |
				status nextPutAll: each name ]
			separatedBy: [ 
				status nextPutAll: ', '].
			status nextPutAll: ' and ', aCollection last name.
			status contents.
		].! !

!MEGAgentUI methodsFor: 'public' stamp: 'SergioFedi 7/4/2016 13:36'!
textForCompany: company
	^ 'Company of ' , (self stringOfNamesOf: company members)! !
